#
#****************************************
#Bandmin 0.41 (c)1998-1999 J. Nick Koston (BlueDraco)
# - A simple Bandwidth Monitor
#****************************************
#
#To use this package you must have a OS that has either ipfwadm,ipchains,
#or ipfw
#

package IP::Acct;

require 5.002;
require Exporter;
use strict;
use Carp;

use vars qw(@ISA $VERSION $IPFWLINENUM $IPACCEPTALL);

$VERSION = 0.10;
$IPFWLINENUM = 1;
$IPACCEPTALL = 2;
@ISA = qw(Exporter);



sub new 
{
	my ($backend);
	my  $self = {};

	bless($self);

	if (-e "/sbin/ipchains") {
		$self->{"backend"} = "ipchains";
	} elsif(-e "/sbin/ipfwadm") {
		$self->{"backend"} = "ipfwadm";		
	} elsif(-e "/sbin/ipfw") {
		$self->{"backend"} = "ipfw";		
	} else {
		die "Can't find a backend (ipchains or ipfwadm or ipfw)";
	}

	return($self);

}

sub flush {
	my ($self) = @_;
	my ($rdt);

	if ($self->{"backend"} eq "ipchains") {
		system("/sbin/ipchains","-N","acctboth");
		system("/sbin/ipchains","-F","acctboth");
	}
	if ($self->{"backend"} eq "ipfwadm") {
		system("/sbin/ipfwadm","-A","-f");
	}
	if ($self->{"backend"} eq "ipfw") {
		$rdt = "";
		while($rdt eq "") {
			$rdt = `/sbin/ipfw delete $IPACCEPTALL 2>&1`;
		}
		system("/sbin/ipfw","add",$IPACCEPTALL,"allow","all","from","any","to","any");
		$rdt = "";
		while($rdt eq "") {
			$rdt = `/sbin/ipfw delete $IPFWLINENUM 2>&1`;
		}
		
	}
}

sub initip {
	my ($self,
	  $ip
	) = @_;

        if ($self->{"backend"} eq "ipchains") {
		system("/sbin/ipchains","-A","acctboth","-s",$ip,"-b","-p","all");
        }
        if ($self->{"backend"} eq "ipfwadm") {
		system("/sbin/ipfwadm","-A","-a","-b","-P","all","-S",$ip);
	}
        if ($self->{"backend"} eq "ipfw") {
		system("/sbin/ipfw","add",$IPFWLINENUM,"allow","all","from","any","to",$ip);
		system("/sbin/ipfw","add",$IPFWLINENUM,"allow","all","from",$ip,"to","any");
	}
}


sub getipbytes {
        my ($self) = @_;

	my($ipdata,$ipstuff,@IPMASS,$ip,$bytes,%ipbytemap,$rule);

        if ($self->{"backend"} eq "ipchains") {
        	$ipstuff = `/sbin/ipchains -L acctboth -v -x -n`;
	}

        if ($self->{"backend"} eq "ipfwadm") {
	        $ipstuff = `/sbin/ipfwadm -Alxn`;
	}

        if ($self->{"backend"} eq "ipfw") {
		$ipstuff = `/sbin/ipfw -a list $IPFWLINENUM`;
	}


	@IPMASS = split(/\n/, $ipstuff);
	foreach $ipdata (@IPMASS) {
        	$ipdata =~ s/\t/ /g;
        	$ipdata =~ s/\s(\s*|.)\s/ /g;
        	$ipdata =~ s/^ //g;
	        if ($self->{"backend"} eq "ipchains") {
			(undef,$bytes,undef,undef,undef,undef,undef,
			undef,undef,$ip,undef) = split(/\s/, $ipdata);
                	if ($ip =~ /\//) {
				(undef,$bytes,undef,undef,undef,undef,
				undef,undef,undef,undef,$ip,
				undef) = split(/\s/, $ipdata);
	                }
	        }
		if ($self->{"backend"} eq "ipfwadm") {
	                (undef,$bytes,undef,undef,$ip,undef,
			undef) = split(/\s/,$ipdata);
	        }
		if ($self->{"backend"} eq "ipfw") {
	                (undef,undef,$bytes,
			$rule) = split(/\s/,$ipdata, 4);
        		if ($rule =~ /(\d*)\.(\d*)\.(\d*)\.(\d*)/) {
                		$ip = "$1.$2.$3.$4";
			} else {
				$ip = "127.0.0.1";
			}
	        }
        	if ($ip =~ /(.*)\.(.*)\.(.*)/) {
                	$ip =~ s/\n//g;
                	$bytes =~ s/\n//g;
			$ipbytemap{$ip} = ($ipbytemap{$ip} + $bytes);
		}
	}
	
	return(%ipbytemap);

}

sub clearcounter {
        my ($self) = @_;
	
        if ($self->{"backend"} eq "ipchains") {
        	system("/sbin/ipchains","-Z","acctboth");
	}

        if ($self->{"backend"} eq "ipfwadm") {
	        system("/sbin/ipfwadm","-Az");
	}

        if ($self->{"backend"} eq "ipfw") {
	        system("/sbin/ipfw","zero",$IPFWLINENUM);
	}


}
